@@ -31,6 +31,7 @@
 #include <X11/StringDefs.h>
 #include <Xw/Xw.h>
 #include <Xw/XwP.h>
+#include <stdlib.h>   /* abs */
 
 
 static int Overlap();
@@ -191,7 +192,7 @@ void mgr_traversal (mgr, ul, lr, direction)
  		     ((Widget)parent, newUl, newLr, direction);
 	 }
          else
-            ClearKbdFocus(mgr);
+            ClearKbdFocus((Widget) mgr);
   	 return;
       }
   
