@@ -219,7 +219,9 @@ extern Cardinal _XwInsertOrder();	   /* compute position from arglist */
  ************************************************************************/
 
 extern void mgr_traversal();
+extern void _XwDrawBox();
 extern void XwSetFocusPath();
+extern void XwMoveFocus();
 extern void XwProcessTraversal();
 extern void XwSetInputFocus();
 extern void _XwInitCheckList();
@@ -251,6 +253,7 @@ extern void _XwInitializeButton();
 extern void _XwRealize();
 extern void _XwResizeButton();
 extern void _XwRegisterName();
+extern Boolean _XwRecomputeSize();
 extern Boolean XwTestTraversability();
 
 /************************************************************************
@@ -320,6 +323,13 @@ typedef struct _XwButtonRec
 #define XtInheritHighlightProc ((XtWidgetProc) _XtInherit)
 #define XtInheritEnterParentProc ((XtWidgetProc) _XtInherit)
 
+/* Corrected by Steve Langasek (was incompatible with the revised Wprintf()) */
+
+typedef void (*XwStrProc)(
+#if NeedFunctionPrototypes
+    char *format, ...  /* hint */
+#endif
+);
 
 /***********************************************************************
  *
@@ -432,6 +442,7 @@ typedef void (*XwWWSProc)(
 
 typedef void (*XwPostProc)(
 #if NeedFunctionPrototypes
+    Widget,     /* widget */
     Widget,     /* widget */
     Widget,     /* widget */
     Position,	/* x */
@@ -461,7 +472,7 @@ extern Boolean _XwMapKeyEvent();
 extern Boolean _XwMatchKeyEvent();
 extern Boolean _XwValidModifier();
 extern String  _XwMapToHex();
-extern int     _XwMapFromHex();
+extern unsigned long _XwMapFromHex();
 
 extern void _XwSetMappedManagedChildrenList();
 
