@@ -8,6 +8,7 @@
 #include <string.h>
 #include <signal.h>
 #include <sys/types.h>
+#include <ctype.h>	/* For isspace() */
 
 #ifdef HAVE_DIRENT_H
 #include <dirent.h>
@@ -75,7 +76,7 @@ extern Tcl_Interp *xcinterp;
 extern Display    *dpy;
 extern XCWindowData *areawin;
 extern ApplicationData appdata;
-extern int 	  *appcolors;
+extern colorindex *colorlist;
 extern short	  popups;     /* total number of popup windows */
 extern char	  _STR2[250];
 extern char	  _STR[150];
@@ -148,7 +149,7 @@ void startfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 #endif
 
    XSetFunction(dpy, areawin->gc, GXcopy);
-   XSetForeground(dpy, areawin->gc, AUXCOLOR);
+   XSetForeground(dpy, areawin->gc, colorlist[AUXCOLOR].color.pixel);
    XSetLineAttributes(dpy, areawin->gc, 1, LineSolid, CapRound, JoinMiter);
 
    /* draw initial box */
@@ -157,7 +158,8 @@ void startfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
    dragfilebox(w, NULL, (XMotionEvent *)event);
 
    XSetFunction(dpy, areawin->gc, GXxor);
-   XSetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+   XSetForeground(dpy, areawin->gc, colorlist[AUXCOLOR].color.pixel ^
+		colorlist[BACKGROUND].color.pixel);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -181,8 +183,8 @@ void endfiletrack(xcWidget w, caddr_t clientdata, XCrossingEvent *event)
 #endif
 
    /* Restore graphics state values */
-   XSetForeground(dpy, areawin->gc, areawin->gccolor);
-   XSetFunction(dpy, areawin->gc, areawin->gctype);
+   XSetForeground(dpy, areawin->gc, colorlist[areawin->gccolor].color.pixel);
+   XSetFunction(dpy, areawin->gc, GXcopy);
 }
 
 #endif
@@ -231,10 +233,15 @@ void crashrecover()
    if (xobjs.tempfile != NULL) {
       unlink(xobjs.tempfile);
       free(xobjs.tempfile);
+      xobjs.tempfile = NULL;
+   }
+   if (strlen(_STR2) == 0) {
+      Wprintf("Error: No temp file name for crash recovery!");
+   }
+   else {
+      xobjs.tempfile = strdup(_STR2);
+      startloadfile(-1);
    }
-   xobjs.tempfile = strdup(_STR2);
-
-   startloadfile(-1);
 }
 
 /*----------------------------------------------------------------------*/
@@ -317,7 +324,7 @@ void findcrashfiles()
 }  
 
 /*----------------------------------------------------------------------*/
-/* Match a filename extension against the file filter string.		*/
+/* Match a filename extension against the file filter list.		*/
 /*----------------------------------------------------------------------*/
 
 #if !defined(XC_WIN32) || defined(TCL_WRAPPER)
@@ -325,11 +332,32 @@ void findcrashfiles()
 Boolean match_filter(char *fname, char *filter)
 {
    char *dotptr = strrchr(fname, '.');
+   char *filtptr, *endptr;
+   int filtlen, extlen;
 
-   if ((dotptr != NULL) && (filter != NULL) &&
-       (filter[0] != '\0') && (strstr(dotptr, filter) != NULL))
-      return True;
+   if (filter == NULL) return False;
+   if (dotptr == NULL) return False;
 
+   /* New 11/08:  Empty string for filter is a wildcard match, the same */
+   /* as turning off the filter.					*/
+
+   if (filter[0] == '\0') return True;
+
+   dotptr++;
+   extlen = strlen(dotptr);
+   filtptr = filter;
+
+   while (*filtptr != '\0') {
+      endptr = filtptr;
+      while (*endptr != '\0' && !isspace(*endptr)) endptr++;
+      filtlen = (int)(endptr - filtptr);
+      if (filtlen == extlen)
+         if (!strncmp(dotptr, filtptr, extlen))
+	    return True;
+
+      filtptr = endptr;
+      while (*filtptr != '\0' && isspace(*filtptr)) filtptr++;
+   }
    return False;
 }
 
@@ -356,7 +384,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
    filter = okaystruct->filter;
 
    if (sgc == NULL) {
-      values.foreground = FOREGROUND;
+      values.foreground = colorlist[FOREGROUND].color.pixel;
       values.font = appdata.filefont->fid;
       values.function = GXcopy;
       values.graphics_exposures = False;
@@ -395,9 +423,9 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
       /* print "Invalid Directory" to the file list window.	  */
 
       if (cwd == NULL) {
-         XSetForeground(dpy, sgc, BACKGROUND);
+         XSetForeground(dpy, sgc, colorlist[BACKGROUND].color.pixel);
          XFillRectangle(dpy, lwin, sgc, 0, 0, textwidth, textheight);
-         XSetForeground(dpy, sgc, AUXCOLOR);
+         XSetForeground(dpy, sgc, colorlist[AUXCOLOR].color.pixel);
          XDrawString(dpy, lwin, sgc, 10, textheight / 2,
 	    "(Invalid Directory)", 19);
 	 return;
@@ -454,19 +482,19 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 
       /* Write the filenames onto the pixmap */
 
-      XSetForeground(dpy, sgc, BACKGROUND);
+      XSetForeground(dpy, sgc, colorlist[BACKGROUND].color.pixel);
       XFillRectangle(dpy, flistpix, sgc, 0, 0, textwidth, pixheight);
-      XSetForeground(dpy, sgc, FOREGROUND);
+      XSetForeground(dpy, sgc, colorlist[FOREGROUND].color.pixel);
       for (n = 0; n < flfiles; n++) {
 	 switch (files[n].filetype) {
 	    case DIRECTORY:
-	       XSetForeground(dpy, sgc, SELECTCOLOR);
+	       XSetForeground(dpy, sgc, colorlist[SELECTCOLOR].color.pixel);
 	       break;
 	    case MATCH:
-	       XSetForeground(dpy, sgc, FILTERCOLOR);
+	       XSetForeground(dpy, sgc, colorlist[FILTERCOLOR].color.pixel);
 	       break;
 	    case NONMATCH:
-	       XSetForeground(dpy, sgc, FOREGROUND);
+	       XSetForeground(dpy, sgc, colorlist[FOREGROUND].color.pixel);
 	       break;
 	 }
          XDrawString(dpy, flistpix, sgc, 10, 10 + FILECHARASCENT + n * FILECHARHEIGHT,
@@ -476,7 +504,7 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 
    /* Copy the pixmap of filenames into the file list window */
 
-   XSetForeground(dpy, sgc, BACKGROUND);
+   XSetForeground(dpy, sgc, colorlist[BACKGROUND].color.pixel);
    XFillRectangle(dpy, lwin, sgc, 0, 0, textwidth, textheight);
    XCopyArea(dpy, flistpix, lwin, sgc, 0, flstart * FILECHARHEIGHT,
 	textwidth, textheight, 0, 0);
@@ -490,12 +518,11 @@ void listfiles(xcWidget w, popupstruct *okaystruct, caddr_t calldata)
 void newfilelist(xcWidget w, popupstruct *okaystruct)
 {
    short n;
-   xcWidget textw = okaystruct->textw;
 
 #ifdef TCL_WRAPPER
    int bval;
    int result;
-   char *cstr = (char *)Tcl_GetVar2(xcinterp, "XCOps", "filter", TCL_NAMESPACE_ONLY);
+   char *cstr = (char *)Tcl_GetVar2(xcinterp, "XCOps", "filter", 0);
    if (cstr == NULL) {
       Wprintf("Error: No variable $XCOps(filter) in Tcl!");
       return;
@@ -506,6 +533,8 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
       return;  
    }
    xobjs.filefilter = bval;
+#else
+   xcWidget textw = okaystruct->textw;
 #endif
 
    for (n = 0; n < flfiles; n++)
@@ -535,26 +564,26 @@ void newfilelist(xcWidget w, popupstruct *okaystruct)
 
 void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
-#ifndef TCL_WRAPPER
-   Arg wargs[2];
-#endif
    Window lwin = xcWindow(w);
    Dimension textwidth, textheight;
-   short filenum, n = 0;
+   short filenum;
    char *tbuf, *ebuf;
-   xcWidget textw = okaystruct->textw;
-
-   flcurrent = -1;
 
 #ifdef TCL_WRAPPER
    textwidth = Tk_Width(w);
    textheight = Tk_Height(w);
 #else
+   Arg wargs[2];
+   short n = 0;
+   xcWidget textw = okaystruct->textw;
+
    XtnSetArg(XtNwidth, &textwidth);
    XtnSetArg(XtNheight, &textheight);
    XtGetValues(w, wargs, n);
 #endif
 
+   flcurrent = -1;
+
    if (files == NULL) return;	/* shouldn't happen */
 
    /* third mouse button cancels selection and reverts buffer to cwd name */
@@ -581,7 +610,7 @@ void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 
       /* highlight the entry. . . */
 
-      XSetForeground(dpy, sgc, AUXCOLOR);
+      XSetForeground(dpy, sgc, colorlist[AUXCOLOR].color.pixel);
       XDrawString(dpy, flistpix, sgc, 10, 10 + FILECHARASCENT + filenum * FILECHARHEIGHT,
    	   files[filenum].filename, strlen(files[filenum].filename));
       XCopyArea(dpy, flistpix, lwin, sgc, 0, flstart * FILECHARHEIGHT,
@@ -659,18 +688,18 @@ void fileselect(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 
 void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
 {
-#ifndef TCL_WRAPPER
-   Arg wargs[2];
-#endif
    Window swin = xcWindow(w);
    Dimension swidth, sheight;
    int pstart, pheight, finscr;
-   short n = 0;
 
 #ifdef TCL_WRAPPER
    swidth = Tk_Width(w);
    sheight = Tk_Height(w);
 #else
+
+   Arg wargs[2];
+   short n = 0;
+
    XtnSetArg(XtNwidth, &swidth);
    XtnSetArg(XtNheight, &sheight);
    XtGetValues(w, wargs, n);
@@ -686,7 +715,7 @@ void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
       pstart = (flstart * sheight) / flfiles;
       pheight = (finscr * sheight) / flfiles;
 
-      XSetForeground(dpy, sgc, BARCOLOR);
+      XSetForeground(dpy, sgc, colorlist[BARCOLOR].color.pixel);
       XFillRectangle(dpy, swin, sgc, 0, pstart, swidth, pheight);
    }
    flcurrent = -1;
@@ -698,17 +727,16 @@ void showlscroll(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
 void draglscroll(xcWidget w, popupstruct *okaystruct, XButtonEvent *event)
 {
-#ifndef TCL_WRAPPER
-   Arg wargs[1];
-#endif
    Dimension sheight;
    int phheight, finscr, flsave = flstart;
-   short n = 0;
    xcWidget filew = okaystruct->filew;
 
 #ifdef TCL_WRAPPER
    sheight = Tk_Height(w);
 #else
+   Arg wargs[1];
+   short n = 0;
+
    XtnSetArg(XtNheight, &sheight);
    XtGetValues(w, wargs, n);
 #endif
