@@ -43,6 +43,10 @@ extern fontinfo *fonts;
 
 extern Globaldata xobjs;
 extern short beeper;
+#ifdef HAVE_CAIRO
+extern XCWindowData *areawin;
+extern const char *utf8encodings[][256];
+#endif
 
 extern float version;
 
@@ -56,12 +60,14 @@ extern Tcl_Interp *xcinterp;
 
 FILE *findfontfile(char *fontname)
 {
-   int i;
+   size_t i;
    char tempname[256];
-   char *tmp_s = getenv((const char *)"XCIRCUIT_LIB_DIR");
    FILE *fd;
 
-   strcpy(_STR, fontname);
+   /* Add subdirectory "fonts".  We will try both with and	*/
+   /* without the subdirectory.					*/
+
+   sprintf(_STR, "fonts/%s", fontname);
 
    /* change string to all lowercase and remove dashes */
 
@@ -72,100 +78,69 @@ FILE *findfontfile(char *fontname)
 
    /* Fprintf(stdout, "Searching for font file \"%s\"\n", _STR); */
 
-   /* pick up file to load from _STR (defined from rcfile.c and files.c) */
-   /* try first in current directory, then in default library and subdir "fonts" */
+   /* Use the mechanism of "libopen" to find the encoding file	*/
+   /* in the search path					*/
 
-   if ((fd = fopen(_STR, "r")) == NULL) {
-      if (tmp_s != NULL) {
-         sprintf(tempname, "%s/%s", tmp_s, _STR);
-         fd = fopen(tempname, "r");
-         if (fd == NULL) {
-            sprintf(tempname, "%s/%s.xfe", tmp_s, _STR);
-            fd = fopen(tempname, "r");
-         }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s", tmp_s, _STR);
-            fd = fopen(tempname, "r");
-         }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s.xfe", tmp_s, _STR);
-            fd = fopen(tempname, "r");
-         }
+   fd = libopen(_STR + 6, FONTENCODING, NULL, 0);
+
+   if (fd == NULL) fd = libopen(_STR, FONTENCODING, NULL, 0);
+
+   /* Some other, probably futile, attempts (call findfontfile recursively) */
+
+   if (fd == NULL) {
+      char *dashptr;
+
+      /* If this font has a suffix, remove it and look for its root */
+      /* font on the supposition that this font is derived from the */
+      /* root font.						    */
+
+      strncpy(tempname, fontname, 99);
+      if ((dashptr = strrchr(tempname, '-')) != NULL) {
+	 *dashptr = '\0';
+	 if ((fd = findfontfile(tempname)) != NULL) return fd;
+
+	 /* And finally, because it's a common case, try adding	*/
+	 /* -Roman and trying again (but don't infinite loop!)	*/
+
+	 if (strcmp(dashptr + 1, "Roman")) {
+	    strcat(dashptr, "-Roman");
+	    if ((fd = findfontfile(tempname)) != NULL) return fd;
+	 }
       }
 
-      /* last resort:  hard-coded directory BUILTINS_DIR and subdir "fonts" */
+      Wprintf("No font encoding file found.");
+      if (fontcount > 0) {	/* Make font substitution */
+	 char *dchr, *psname = NULL;
+	 short fval;
 
-      if (fd == NULL) {
-         sprintf(tempname, "%s/%s", BUILTINS_DIR, _STR);
-         fd = fopen(tempname, "r");
-         if (fd == NULL) {
-            sprintf(tempname, "%s/%s.xfe", BUILTINS_DIR, _STR);
-            fd = fopen(tempname, "r");
+	 if ((dchr = strrchr(_STR, '.')) != NULL) *dchr = '\0';
+	 if ((fval = findhelvetica()) == fontcount) {
+	    /* This will cause some chaos. . . */
+	    Fprintf(stderr, "Error:  No fonts available!  Check library path?\n");
+	    exit(1);
 	 }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s", BUILTINS_DIR, _STR);
-            fd = fopen(tempname, "r");
-	 }
-         if (fd == NULL) {
-            sprintf(tempname, "%s/fonts/%s.xfe", BUILTINS_DIR, _STR);
-            fd = fopen(tempname, "r");
-	 }
-         if (fd == NULL) {
-	    char *dashptr;
 
-	    /* If this font has a suffix, remove it and look for its root */
-	    /* font on the supposition that this font is derived from the */
-	    /* root font.						  */
-
-	    strncpy(tempname, fontname, 99);
-	    if ((dashptr = strrchr(tempname, '-')) != NULL) {
-	       *dashptr = '\0';
-	       if ((fd = findfontfile(tempname)) != NULL) return fd;
-
-	       /* And finally, because it's a common case, try adding	*/
-	       /* -Roman and trying again (but don't infinite loop!)	*/
-
-	       if (strcmp(dashptr + 1, "Roman")) {
-	          strcat(dashptr, "-Roman");
-	          if ((fd = findfontfile(tempname)) != NULL) return fd;
-	       }
-	    }
-
-            Wprintf("No font encoding file found.");
-	    if (fontcount > 0) {	/* Make font substitution */
-	       char *dchr, *psname = NULL;
-	       short fval;
-
-	       if ((dchr = strrchr(_STR, '.')) != NULL) *dchr = '\0';
-	       if ((fval = findhelvetica()) == fontcount) {
-		  /* This will cause some chaos. . . */
-		  Fprintf(stderr,
-			  "Error:  No fonts available!  Check library path?\n");
-		  exit(1);
-	       }
-	       psname = (char *)malloc((1 + strlen(fontname)) * sizeof(char));
-	       strcpy(psname, fontname);
-	       Wprintf("No encoding file found for font %s: substituting %s",
+	 psname = (char *)malloc((1 + strlen(fontname)) * sizeof(char));
+	 strcpy(psname, fontname);
+	 Wprintf("No encoding file found for font %s: substituting %s",
 			psname, fonts[fval].psname);
-               fonts = (fontinfo *)realloc(fonts, (fontcount + 1) * sizeof(fontinfo));
-               fonts[fontcount].psname = psname;
-               fonts[fontcount].family = psname;
-               fonts[fontcount].encoding = fonts[fval].encoding;
-               fonts[fontcount].flags = 0; 
-               fonts[fontcount].scale = 1.0;
-               fontcount++;
-	       makenewfontbutton();
-	    }
-	    else {
-	       Fprintf(stderr, "Error:  font encoding file missing for font \"%s\"\n",
-			fontname);
-	       Fprintf(stderr, "No fonts exist for a subsitution.  Make sure "
-			"fonts are installed or that\nenvironment variable "
-			"XCIRCUIT_LIB_DIR points to a directory of valid fonts.\n");
-	    }
-            return (FILE *)NULL;
-         }
+         fonts = (fontinfo *)realloc(fonts, (fontcount + 1) * sizeof(fontinfo));
+         fonts[fontcount].psname = psname;
+         fonts[fontcount].family = psname;
+         fonts[fontcount].encoding = fonts[fval].encoding;
+         fonts[fontcount].flags = 0; 
+         fonts[fontcount].scale = 1.0;
+         fontcount++;
+	 makenewfontbutton();
       }
+      else {
+	 Fprintf(stderr, "Error:  font encoding file missing for font \"%s\"\n",
+		fontname);
+	 Fprintf(stderr, "No fonts exist for a subsitution.  Make sure "
+		"fonts are installed or that\nenvironment variable "
+		"XCIRCUIT_LIB_DIR points to a directory of valid fonts.\n");
+      }
+      return (FILE *)NULL;
    }
    return fd;
 }
@@ -176,7 +151,7 @@ FILE *findfontfile(char *fontname)
 /* other error occurred.						*/
 /*----------------------------------------------------------------------*/
 
-int loadfontfile(char *fontname)
+int loadfontfile(char *fname)
 {
    FILE *fd;
    char temp[250], commandstr[30], tempname[100];
@@ -188,11 +163,16 @@ int loadfontfile(char *fontname)
    objectptr *j, *eptr;
    objectptr *encoding = NULL;
    float saveversion = version;
+   char *fontname = strdup(fname);
+#ifdef HAVE_CAIRO
+   const char **utf8enc = NULL;
+#endif /* HAVE_CAIRO */
 
    /* check to see if font name is in list of fonts */
 
    for (i = 0; i < fontcount; i++) {
       if (!strcmp(fonts[i].psname, fontname)) {
+	 free(fontname);
 	 return 0;
       }
    }
@@ -217,6 +197,7 @@ int loadfontfile(char *fontname)
             if (!strcmp(fonts[i].psname, tempname)) {
 	       /* Fprintf(stdout, "Font already loaded.\n"); */
 	       fclose(fd);
+	       free(fontname);
 	       return 0;
 	    }
 	 }
@@ -232,7 +213,8 @@ int loadfontfile(char *fontname)
 	 /* current version number for the file load.			*/
 
 	 version = PROG_VERSION;
-	 loadlibrary(FONTLIB);
+	 if (loadlibrary(FONTLIB) == FALSE) {
+	 };
 	 version = saveversion;
       }
 
@@ -285,8 +267,9 @@ int loadfontfile(char *fontname)
 
 	    sscanf(temp, "%*s %*s %99s", tempname);
 	    tempname[0] = tolower(tempname[0]);
-	    if (!strcmp(tempname, "bold"))
+	    if (!strcmp(tempname, "bold")) {
 	       flags |= 0x1;
+	    }
 
 	    sscanf(temp, "%*s %*s %*s %99s", tempname);
 	    tempname[0] = tolower(tempname[0]);
@@ -311,33 +294,57 @@ int loadfontfile(char *fontname)
             fonts[fontcount].encoding = encoding;  /* use original encoding */
             fonts[fontcount].flags = flags; 
 	    fonts[fontcount].scale = fontscale;
+#ifdef HAVE_CAIRO
+	    fonts[fontcount].utf8encoding = utf8enc;
+	    xc_cairo_set_fontinfo(fontcount);
+#endif /* HAVE_CAIRO */
             fontcount++;
 	 }
       }
 
       else if (!strcmp(cmdptr, "encoding:")) {
+	 #ifdef HAVE_CAIRO
+	 const char *findencstr;
+         #endif /* HAVE_CAIRO */
 	 sscanf(temp, "%*s %99s", tempname);
-	 
+
 	 if (!strcmp(tempname, "special") || !strcmp(tempname, "Special")) {
 	    flags |= 0x80;
 #ifdef TCL_WRAPPER
-	    Tcl_Eval(xcinterp, "xcircuit::newencodingbutton special");
+	    XcInternalTagCall(xcinterp, 3, "label", "encoding", "Special");
 #else
 	    makenewencodingbutton("Special", (char)1);
 #endif
 	 }
+	
+#ifdef HAVE_CAIRO
+	    utf8enc = utf8encodings[0];
+	    /* treat encoding of symbol font in a special way */
+	    if (!strcmp(family, "Symbol"))
+	       findencstr = "Symbol";
+	    else
+	       findencstr = tempname;
+
+	    /* Now try to find the encoding, */
+	    /* default to the first encoding (stdenc) if not found */
+	    utf8enc = utf8encodings[0];
+	    for (i = 0; utf8encodings[i][0]; i++)
+	       if (!strcmp(utf8encodings[i][0], findencstr)) {
+		  utf8enc = utf8encodings[i];
+		  break;
+	       }
+#endif /* HAVE_CAIRO */
 
 	 /* ISO-LatinX encodings where X=1 to 6 */
 
-	 if (strstr(tempname, "ISO") != NULL) {
-	    char estr[38];
+	 if (strstr(tempname, "Latin") != NULL) {
+	    char estr[12];
 	    for (i = 0; i < 6; i++) {
 	       if (strchr(tempname, '1' + (char)i) != NULL) {
 	          flags |= ((i + 2) << 7);
 #ifdef TCL_WRAPPER
-		  snprintf(estr, sizeof(estr),
-			"xcircuit::newencodingbutton ISOLatin%d", i + 1);
-		  Tcl_Eval(xcinterp, estr);
+		  snprintf(estr, sizeof(estr), "ISOLatin%d", i + 1);
+		  XcInternalTagCall(xcinterp, 3, "label", "encoding", estr);
 #else
 		  snprintf(estr, sizeof(estr), "ISO-Latin%d", i + 1);
 	          makenewencodingbutton(estr, (char)(i + 2));
@@ -346,6 +353,19 @@ int loadfontfile(char *fontname)
 	       }
 	    }
 	 }
+	 /* i == 6 maps to ISO8859-5 */
+	 if (strstr(tempname, "8859-5") != NULL) {
+#ifndef TCL_WRAPPER
+	    char estr[12];
+#endif
+	    flags |= ((6 + 2) << 7);
+#ifdef TCL_WRAPPER
+	    XcInternalTagCall(xcinterp, 3, "label", "encoding", "ISO8859-5");
+#else
+	    snprintf(estr, sizeof(estr), "ISO-8859-5");
+	    makenewencodingbutton(estr, (char)8);
+#endif
+	 }
 
          /* Make space for the font encoding vector */
 
@@ -394,6 +414,10 @@ int loadfontfile(char *fontname)
          fonts[fontcount].encoding = encoding;
          fonts[fontcount].flags = flags; 
          fonts[fontcount].scale = fontscale;
+#ifdef HAVE_CAIRO
+	 fonts[fontcount].utf8encoding = utf8enc;
+	 xc_cairo_set_fontinfo(fontcount);
+#endif /* HAVE_CAIRO */
          fontcount++;
 
          /* Create a new menu button for the font family, if this is the first	*/
@@ -409,5 +433,6 @@ int loadfontfile(char *fontname)
       }
    }
    fclose(fd);
+   free(fontname);
    return 1;
 }
