@@ -56,8 +56,8 @@ extern Display    *dpy;
 extern GC	  hgc;
 extern XCWindowData *areawin;
 extern ApplicationData appdata;
-extern int 	  *appcolors;
 extern xcWidget     top;
+extern colorindex *colorlist;
 extern short	  popups;
 extern xcWidget	  menuwidgets[];
 extern char *function_names[NUM_FUNCTIONS];
@@ -131,7 +131,7 @@ void printhelppix()
     { XCF_Dotted,	"Dotted line style"},
     { XCF_Solid,	"Solid line style"},
     { XCF_SPACER,	NULL},
-    { XCF_Justify,	"Text Justification"},
+    { XCF_Anchor,	"Text Anchoring"},
     { XCF_Superscript,	"Text Superscript"},
     { XCF_Subscript,	"Text Subscript"},
     { XCF_Font,		"Toggle text font"},
@@ -156,11 +156,11 @@ void printhelppix()
     { XCF_Text_Up,	"Move up one line"},
     { XCF_Text_Down,	"Move down one line"},
     { XCF_Text_Delete,	"Delete character(s)"},
+    { XCF_Text_Delete_Param,	"Delete parameter from text"},
     { XCF_Text_Return,	"End text edit"},
     { XCF_Text_Split,	"Split label at cursor"},
     { XCF_SPACER,	NULL},
     { XCF_Edit_Next,	"Edit: next position"},
-    { XCF_Edit_Break,	"Edit: break line at point"},
     { XCF_Edit_Delete,	"Edit: delete point"},
     { XCF_Edit_Insert,	"Edit: insert point"},
     { XCF_Edit_Param,	"Edit: insert parameter"},
@@ -196,8 +196,8 @@ void printhelppix()
     /* Set up the GC for drawing to the help window pixmap */
 
     if (hgc == NULL) {
-       values.foreground = FOREGROUND;
-       values.background = BACKGROUND;
+       values.foreground = colorlist[FOREGROUND].color.pixel;
+       values.background = colorlist[BACKGROUND].color.pixel;
        values.graphics_exposures = False;
        values.font = appdata.helpfont->fid;
        hgc = XCreateGC(dpy, hwin, GCForeground | GCBackground | GCFont
@@ -261,10 +261,10 @@ void printhelppix()
     helppix = XCreatePixmap(dpy, hwin, helpwidth, helpheight,
 	   	  DefaultDepthOfScreen(DefaultScreenOfDisplay(dpy)));
 
-    XSetForeground(dpy, hgc, FOREGROUND);
+    XSetForeground(dpy, hgc, colorlist[FOREGROUND].color.pixel);
     XFillRectangle(dpy, helppix, hgc, 0, 0, helpwidth, helpheight);
 
-    XSetForeground(dpy, hgc, BACKGROUND);
+    XSetForeground(dpy, hgc, colorlist[BACKGROUND].color.pixel);
     XDrawString(dpy, helppix, hgc, (helpwidth - XTextWidth(appdata.helpfont,
 	helptitle, strlen(helptitle))) >> 1, t1 + 2, helptitle, strlen(helptitle));
     vtmp = lineheight + 15;
@@ -291,7 +291,7 @@ void printhelppix()
        }
        free(bindings);
     }
-    XSetForeground(dpy, hgc, AUXCOLOR);
+    XSetForeground(dpy, hgc, colorlist[AUXCOLOR].color.pixel);
     XDrawLine(dpy, helppix, hgc, 0, t1 + 7, helpwidth, t1 + 7);
 }
 
@@ -459,7 +459,7 @@ void showhsb(xcWidget hsb, caddr_t clientdata, caddr_t calldata)
    XClearArea(dpy, hwin, 0, pstart + pheight, SBARSIZE,
 		hheight - (pstart + pheight), False);
 
-   XSetForeground(dpy, hgc, BARCOLOR);
+   XSetForeground(dpy, hgc, colorlist[BARCOLOR].color.pixel);
    XFillRectangle(dpy, hwin, hgc, 0, pstart, SBARSIZE, pheight);
 }
 
