@@ -86,7 +86,12 @@ typedef struct {
    float scale;			/* old scale value */
 } scaleinfo;
 
-Boolean undo_collect = FALSE;
+typedef struct {
+   XPoint rotpos;		/* original position */
+   float rotation;		/* old rotation value */
+} rotateinfo;
+
+u_char undo_collect = (u_char)0;
 
 /*----------------------------------------------------------------------*/
 /* Externally declared variables					*/
@@ -418,7 +423,8 @@ void undelete_one_element(objinstptr thisinst, genericptr thiselem)
 void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 {
    va_list args;
-   int drawmode, nval, oval, *idata, snum, deltax, deltay, dir, i;
+   int drawmode, nval, oval, *idata, snum, deltax, deltay, i;
+   double dir;
    short *slist;
    objectptr delobj;
    objinstptr newinst;
@@ -458,7 +464,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
    else
       newrecord->idx = 1;
 
-   if (mode == UNDO_MORE || undo_collect == TRUE)
+   if (mode == UNDO_MORE || undo_collect > (u_char)0)
       newrecord->idx = -newrecord->idx;
    
    xobjs.undostack = newrecord;
@@ -541,7 +547,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	       erec->save.string =
 			stringcopyall(((labelptr)egen)->string,
 			areawin->topinstance);
-	       newrecord->idata = ((labelptr)egen)->justify;
+	       newrecord->idata = ((labelptr)egen)->anchor;
 	       break;
 	    case POLYGON:
 	       newrecord->idata = ((polyptr)egen)->number;
@@ -589,7 +595,7 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
 	 break;
 
       case XCF_ChangeStyle:
-      case XCF_Justify:
+      case XCF_Anchor:
       case XCF_Color:
 	 /* 2 args:							*/
 	 /*	egen = element that was changed (with new value)	*/
@@ -627,13 +633,13 @@ void register_for_undo(u_int type, u_char mode, objinstptr thisinst, ...)
       case XCF_Rotate:
 	 /* 2 args:							*/
 	 /*	fpoint = point of flip (XPoint *)			*/
-	 /*	dir = direction and amound of rotation (int)		*/
+	 /*	dir = direction and amount of rotation (float)		*/
 	 fpoint = va_arg(args, XPoint *);
-	 dir = va_arg(args, int);
-	 newrecord->undodata = (char *)malloc(sizeof(XPoint));
-	 ((XPoint *)newrecord->undodata)->x = fpoint->x;
-	 ((XPoint *)newrecord->undodata)->y = fpoint->y;
-	 newrecord->idata = dir;
+	 dir = va_arg(args, double);
+	 newrecord->undodata = (char *)malloc(sizeof(rotateinfo));
+	 ((rotateinfo *)newrecord->undodata)->rotpos.x = fpoint->x;
+	 ((rotateinfo *)newrecord->undodata)->rotpos.y = fpoint->y;
+	 ((rotateinfo *)newrecord->undodata)->rotation = (float)dir;
 	 break;
 
       case XCF_Move:
@@ -747,8 +753,7 @@ short undo_one_action()
 	    break;
 	 }
 	 else {
-	    /* XXX egen is uninitialized! */
-	    delete_one_element(thisrecord->thisinst, egen);
+	    delete_one_element(thisrecord->thisinst, (genericptr)thisinst);
 	 }
 
 	 /* Put back all the parts */
@@ -774,11 +779,12 @@ short undo_one_action()
 	 newpage(thisrecord->idata);
 	 break;
 
-      case XCF_Justify:
+      case XCF_Anchor:
 	 thislabel = (labelptr)(thisrecord->undodata);
 	 snum = thisrecord->idata;
-	 thisrecord->idata = thislabel->justify;
-	 thislabel->justify = snum;
+	 thisrecord->idata = thislabel->anchor;
+	 thislabel->anchor = snum;
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -789,8 +795,10 @@ short undo_one_action()
 	 
 	 need_redraw = (areawin->selects > 0) ? True : False;
 	 select_previous(thisrecord);
-	 if (need_redraw)
+	 if (need_redraw) {
+	    areawin->redraw_needed = True;
 	    drawarea(areawin->area, NULL, NULL);
+	 }
 	 else
 	    draw_all_selected();
 	 break;
@@ -806,6 +814,7 @@ short undo_one_action()
 	 }
 	 else {
 	    delete_one_element(thisrecord->thisinst, egen);
+	    areawin->redraw_needed = True;
 	    drawarea(areawin->area, NULL, NULL);
 	 }
 	 break;
@@ -815,16 +824,16 @@ short undo_one_action()
 	 switch (erec->element->type) {
 	    case LABEL: {
 	       stringpart *tmpstr;
-	       int tmpjust;
+	       int tmpanchor;
 	       labelptr elab = (labelptr)(erec->element);
 	       undrawtext(elab);
 	       tmpstr = elab->string;
-	       tmpjust = (int)elab->justify;
+	       tmpanchor = (int)elab->anchor;
 	       elab->string = stringcopyback(erec->save.string,
 				thisrecord->thisinst);
-	       elab->justify = (short)thisrecord->idata;
+	       elab->anchor = (short)thisrecord->idata;
 	       erec->save.string = tmpstr;
-	       thisrecord->idata = tmpjust;
+	       thisrecord->idata = tmpanchor;
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
@@ -843,14 +852,17 @@ short undo_one_action()
 	       earc->position = erec->save.arcspecs->position;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case OBJINST: {
 	       XPoint tmppt;
 	       objinstptr einst = (objinstptr)(erec->element);
+	       // Swap instance and saved positions.
 	       tmppt = einst->position;
 	       einst->position = erec->save.instpos;
-	       erec->save.instpos = ((objinstptr)egen)->position;
+	       erec->save.instpos = tmppt;
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    }  break;
 	    case POLYGON: {
@@ -863,6 +875,7 @@ short undo_one_action()
 	       epoly->number = thisrecord->idata;
 	       erec->save.points = tmppts;
 	       thisrecord->idata = tmpnum;
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case SPLINE: {
@@ -874,6 +887,7 @@ short undo_one_action()
 	       free(erec->save.points);
 	       erec->save.points = tmppts;
 	       calcspline(espline);
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case PATH: {
@@ -905,6 +919,7 @@ short undo_one_action()
 			break;
 		  }
 	       }
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
@@ -930,7 +945,9 @@ short undo_one_action()
 
 	 /* Revert selection to previously selected */
 	 select_previous(thisrecord);
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
+	 draw_all_selected();
 	 break;
 
       case XCF_ChangeStyle:
@@ -959,6 +976,7 @@ short undo_one_action()
 	       thisspline->style = snum;
 	       break;
 	 }
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -988,6 +1006,7 @@ short undo_one_action()
 	       thisspline->color = snum;
 	       break;
 	 }
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1025,7 +1044,9 @@ short undo_one_action()
 	       thisgraphic = (graphicptr)egen;
 	       escale->scale = thisgraphic->scale;
 	       thisgraphic->scale = fnum;
+#ifndef HAVE_CAIRO
 	       thisgraphic->valid = FALSE;
+#endif /* HAVE_CAIRO */
 	       break;
 	    case LABEL:
 	       thislabel = (labelptr)egen;
@@ -1033,6 +1054,7 @@ short undo_one_action()
 	       thislabel->scale = fnum;
 	       break;
 	 }
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1047,8 +1069,8 @@ short undo_one_action()
 	 break;
 
       case XCF_Rotate:
-	 position = *((XPoint *)thisrecord->undodata);
-	 elementrotate(-thisrecord->idata, &position);
+	 position = ((rotateinfo *)thisrecord->undodata)->rotpos;
+	 elementrotate(-((rotateinfo *)thisrecord->undodata)->rotation, &position);
 	 break;
 
       case XCF_Move:
@@ -1056,11 +1078,14 @@ short undo_one_action()
 	 select_connected_pins();
 	 placeselects(-(delta->x), -(delta->y), NULL);
 	 reset_cycles();
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
+	 draw_all_selected();
 	 break;
 
        case XCF_Reorder:
 	 reorder_selection(thisrecord);
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1094,12 +1119,12 @@ short undo_one_action()
 /*	Complete a possibly incomplete undo series by forcing the	*/
 /*	topmost entry to have a positive index.				*/
 /*	Note that for "undo series start|end" to work, undo_collect	*/
-/*	must be set to FALSE prior to calling undo_finish_series().	*/
+/*	must be set to 0 prior to calling undo_finish_series().		*/
 /*----------------------------------------------------------------------*/
 
 void undo_finish_series()
 {
-   if (undo_collect == FALSE)
+   if (undo_collect == (u_char)0)
       if (xobjs.undostack && xobjs.undostack->idx < 0)
          xobjs.undostack->idx = -xobjs.undostack->idx;
 }
@@ -1111,9 +1136,14 @@ void undo_finish_series()
 
 void undo_action()
 {
-   short idx = undo_one_action(False);
+   short idx;
+
+   // Cannot undo while in the middle of an undo series.  Failsafe.
+   if (undo_collect != (u_char)0) return;
+
+   idx = undo_one_action();
    while (xobjs.undostack && xobjs.undostack->idx == idx)
-      undo_one_action(False);
+      undo_one_action();
 }
 
 /*----------------------------------------------------------------------*/
@@ -1175,6 +1205,7 @@ short redo_one_action()
 	 thisrecord->undodata = (char *)thisobj;
 	 thisrecord->idata = (int)DRAW;
 	 unselect_all();
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1188,11 +1219,12 @@ short redo_one_action()
 	 newpage(*((int *)thisrecord->undodata));
 	 break;
 
-      case XCF_Justify:
+      case XCF_Anchor:
 	 thislabel = (labelptr)(thisrecord->undodata);
 	 snum = thisrecord->idata;
-	 thisrecord->idata = thislabel->justify;
-	 thislabel->justify = snum;
+	 thisrecord->idata = thislabel->anchor;
+	 thislabel->anchor = snum;
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1242,6 +1274,7 @@ short redo_one_action()
       case XCF_Spline: case XCF_Dot: case XCF_Graphic: case XCF_Join:
 	 egen = (genericptr)thisrecord->undodata;
 	 undelete_one_element(thisrecord->thisinst, egen);
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1250,25 +1283,27 @@ short redo_one_action()
 	 switch (erec->element->type) {
 	    case LABEL: {
 	       stringpart *tmpstr;
-	       int tmpjust;
+	       int tmpanchor;
 	       labelptr elab = (labelptr)(erec->element);
 	       undrawtext(elab);
 	       tmpstr = elab->string;
-	       tmpjust = (int)elab->justify;
+	       tmpanchor = (int)elab->anchor;
 	       elab->string = stringcopyback(erec->save.string,
 				thisrecord->thisinst);
-	       elab->justify = (short)thisrecord->idata;
+	       elab->anchor = (short)thisrecord->idata;
 	       erec->save.string = tmpstr;
-	       thisrecord->idata = tmpjust;
+	       thisrecord->idata = tmpanchor;
 	       resolveparams(thisrecord->thisinst);
 	       redrawtext(elab);
 	    } break;
 	    case OBJINST: {
 	       XPoint tmppt;
 	       objinstptr einst = (objinstptr)(erec->element);
+	       // Swap instance position and saved position
 	       tmppt = einst->position;
 	       einst->position = erec->save.instpos;
-	       erec->save.instpos = ((objinstptr)egen)->position;
+	       erec->save.instpos = tmppt;
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    }  break;
 	    case ARC: {
@@ -1286,6 +1321,7 @@ short redo_one_action()
 	       earc->position = erec->save.arcspecs->position;
 	       *(erec->save.arcspecs) = tmpinfo;
 	       calcarc(earc);
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case POLYGON: {
@@ -1298,6 +1334,7 @@ short redo_one_action()
 	       epoly->number = thisrecord->idata;
 	       erec->save.points = tmppts;
 	       thisrecord->idata = tmpnum;
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case SPLINE: {
@@ -1309,6 +1346,7 @@ short redo_one_action()
 	       free(erec->save.points);
 	       erec->save.points = tmppts;
 	       calcspline(espline);
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    } break;
 	    case PATH: {
@@ -1340,6 +1378,7 @@ short redo_one_action()
 			break;
 		  }
 	       }
+	       areawin->redraw_needed = True;
 	       drawarea(areawin->area, NULL, NULL);
 	    }
 	 }
@@ -1381,6 +1420,7 @@ short redo_one_action()
 	       thisspline->style = snum;
 	       break;
 	 }
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1410,6 +1450,7 @@ short redo_one_action()
 	       thisspline->color = snum;
 	       break;
 	 }
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1447,7 +1488,9 @@ short redo_one_action()
 	       thisgraphic = (graphicptr)egen;
 	       escale->scale = thisgraphic->scale;
 	       thisgraphic->scale = fnum;
+#ifndef HAVE_CAIRO
 	       thisgraphic->valid = FALSE;
+#endif /* HAVE_CAIRO */
 	       break;
 	    case LABEL:
 	       thislabel = (labelptr)egen;
@@ -1455,12 +1498,13 @@ short redo_one_action()
 	       thislabel->scale = fnum;
 	       break;
 	 }
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       case XCF_Rotate:
-	 position = *((XPoint *)thisrecord->undodata);
-	 elementrotate(thisrecord->idata, &position);
+	 position = ((rotateinfo *)thisrecord->undodata)->rotpos;
+	 elementrotate(((rotateinfo *)thisrecord->undodata)->rotation, &position);
 	 break;
 
       case XCF_Move:
@@ -1468,11 +1512,13 @@ short redo_one_action()
 	 select_connected_pins();
 	 placeselects(delta->x, delta->y, NULL);
 	 reset_cycles();
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
       case XCF_Reorder:
 	 reorder_selection(thisrecord);
+	 areawin->redraw_needed = True;
 	 drawarea(areawin->area, NULL, NULL);
 	 break;
 
@@ -1504,7 +1550,12 @@ short redo_one_action()
 
 void redo_action()
 {
-   short idx = redo_one_action();
+   short idx;
+
+   // Cannot redo while in the middle of an undo series.  Failsafe.
+   if (undo_collect != (u_char)0) return;
+
+   idx = redo_one_action();
    while (xobjs.redostack && xobjs.redostack->idx == idx)
       redo_one_action();
 }
@@ -1615,7 +1666,7 @@ void free_undo_data(Undoptr thisrecord, u_char mode)
 
       case XCF_Push:
       case XCF_ChangeStyle:
-      case XCF_Justify:
+      case XCF_Anchor:
       case XCF_Color:
 	 /* Do nothing --- undodata points to a valid element */
 	 break;
